unit UAsmSprite;

interface

uses
	Windows, Classes, Graphics, SysUtils;

////////////////////////////////////////////////////////////////////////////////
// Interface of TAssembledSprite
////////////////////////////////////////////////////////////////////////////////

type
	TAssembledSprite = class
	public
		constructor Create(const srcImage: TBitmap; const Id: string; colorKey: word);
		procedure Free();
	public
		spriteCode: TStrings;
	end;

implementation

////////////////////////////////////////////////////////////////////////////////
// Implementation of TAssembledSprite
////////////////////////////////////////////////////////////////////////////////

constructor TAssembledSprite.Create(const srcImage: TBitmap; const Id: string; colorKey: word);
var
	i          : integer;
	spr_width  : integer;
	spr_height : integer;
	spr_bpp    : integer;

	x, y       : integer;
	currentFlag: (cfSkip, cfCopy);
	pBeacon    : Pword;
	pDest16    : Pword;
	skip       : integer;
	count      : integer;
	sTemp      : string;
	nLabel     : integer;
	labelCode  : TStrings;
begin
	spr_width  := srcImage.Width;
	spr_height := srcImage.Height;
	spr_bpp    := sizeof(word);
	count      := 0;

	spriteCode := TStringList.Create();
	labelCode  := TStringList.Create();

	// declarations begin
	spriteCode.Add(Format('procedure DrawSprite%s(const pAddr: pointer; const pitch: longint); register; assembler;', [Id]));
	spriteCode.Add('asm');

	// initialization
	spriteCode.Add(^I'jmp  @@START');
	spriteCode.Add(Format(^I'dd   $%0.8x, $%0.8x', [spr_width, spr_height]));
	spriteCode.Add('');
	spriteCode.Add('@@START:');
	spriteCode.Add(^I'push edi');
	spriteCode.Add(^I'push esi');
	spriteCode.Add('');
	spriteCode.Add(^I'mov  edi, eax');
	spriteCode.Add(Format(^I'sub  edx, %d*%d', [spr_width, spr_bpp]));
	spriteCode.Add('');

	nLabel := 0;
	pBeacon := nil;
	for y := 0 to pred(spr_height) do begin
		pDest16 := srcImage.ScanLine[y];
		currentFlag := cfSkip;
		skip  := 0;
		for x := 0 to pred(spr_width) do begin
			case currentFlag of
				cfSkip:
				begin
					if pDest16^ = colorKey then begin
						inc(skip);
					end
					else begin
						if skip > 0 then begin
							// skipping
							spriteCode.Add(Format(^I'add  edi, %d*%d', [skip, spr_bpp]));
							spriteCode.Add('');
						end;
						currentFlag := cfCopy;
						pBeacon := pDest16;
						count := 1;
					end;
				end;
				cfCopy:
				begin
					if pDest16^ <> colorKey then begin
						inc(count);
					end
					else begin
						sTemp := Format('@@DATA%0.6d', [nLabel]);
						inc(nLabel);
						// copying
						spriteCode.Add(Format(^I'lea  esi, %s', [sTemp]));
						spriteCode.Add(Format(^I'mov  ecx, %d', [count]));
						spriteCode.Add(^I'rep  movsw');
						spriteCode.Add('');

						labelCode.Add(Format('%s:', [sTemp]));
						sTemp := ^I'dw   ';
						while count > 0 do begin
							sTemp := sTemp + Format('$%0.4x', [pBeacon^]);
							inc(pBeacon);
							dec(count);
							if count > 0 then
								sTemp := sTemp + ', ';
						end;
						labelCode.Add(sTemp);

						currentFlag := cfSkip;
						skip  := 1;
					end;
				end;
			end;
			inc(pDest16);
		end;

		case currentFlag of
			cfSkip:
			begin
				if skip > 0 then begin
					// skipping
					spriteCode.Add(Format(^I'add  edi, %d*%d', [skip, spr_bpp]));
					spriteCode.Add('');
				end;
			end;
			cfCopy:
			begin
				sTemp := Format('@@DATA%0.6d', [nLabel]);
				inc(nLabel);
				// copying
				spriteCode.Add(Format(^I'lea  esi, %s', [sTemp]));
				spriteCode.Add(Format(^I'mov  ecx, %d', [count]));
				spriteCode.Add(^I'rep  movsw');
				spriteCode.Add('');

				labelCode.Add(Format('%s:', [sTemp]));
				sTemp := ^I'dw   ';
				while count > 0 do begin
					sTemp := sTemp + Format('$%0.4x', [pBeacon^]);
					inc(pBeacon);
					dec(count);
					if count > 0 then
						sTemp := sTemp + ', ';
				end;
				labelCode.Add(sTemp);
			end;
		end;

		// line feed and carriage return
		spriteCode.Add(^I'add  edi, edx');
		spriteCode.Add('');
	end;

	// finalization
	spriteCode.Add(^I'pop  esi');
	spriteCode.Add(^I'pop  edi');
	spriteCode.Add('');
	spriteCode.Add(^I'ret');
	spriteCode.Add('');

	// filling out data section
	for i := 0 to pred(labelCode.Count) do begin
		spriteCode.Add(labelCode[i]);
	end;

	// declaration ends
	spriteCode.Add('end;');

	labelCode.Free;
end;

procedure TAssembledSprite.Free();
begin
	if assigned(spriteCode) then
		spriteCode.Free;
end;

end.
 