unit UFrameBuffer;

interface

uses
	Windows, Graphics;

////////////////////////////////////////////////////////////////////////////////
// Interface of TFrameBuffer
////////////////////////////////////////////////////////////////////////////////

type
	TSpriteProc = procedure(const pAddr: pointer; const pitch: longint); register;

	TFrameBuffer = class(TBitmap)
	public
		constructor Create(w, h: integer);

	private
		m_pSurface: pointer;
		m_pitch: longint;

	public
		procedure DrawImage(x, y: integer; const proc: TSpriteProc); overload;
		procedure DrawImage(x, y: integer; const bitmap: TBitmap; const colorKey: word); overload;
		procedure Flush(const canvas: TCanvas; x, y: integer);

	end;

implementation

////////////////////////////////////////////////////////////////////////////////
// Implementation of TFrameBuffer
////////////////////////////////////////////////////////////////////////////////

constructor TFrameBuffer.Create(w, h: integer);
begin
	inherited Create();

	Self.Width  := w;
	Self.Height := h;
	Self.PixelFormat := pf15Bit;
	Self.Canvas.Brush.Color := RGB(255, 255, 255);
	Self.Canvas.FillRect(Self.Canvas.ClipRect);

	m_pSurface := Self.ScanLine[0];
	m_pitch    := longint(Self.ScanLine[1]) - longint(Self.ScanLine[0]);
end;

procedure TFrameBuffer.DrawImage(x, y: integer; const proc: TSpriteProc); register; assembler;
asm
	push ebx
	xchg edx, ecx
	mov  ebx, eax
	mov  eax, [ebx+m_pitch]
	imul eax, edx
	mov  edx, [ebx+m_pSurface]
	add  eax, edx;
	lea  eax, [eax+ecx*2]
	mov  edx, [ebx+m_pitch]
	call proc
	pop  ebx
end;

procedure TFrameBuffer.DrawImage(x, y: integer; const bitmap: TBitmap; const colorKey: word);
var
	i, j: integer;
	pSour16: Pword;
	pDest16: Pword;
begin
	for j := 0 to pred(bitmap.Height) do begin
		pSour16 := bitmap.ScanLine[j];
		pDest16 := Self.ScanLine[y+j];
		inc(pDest16, x);
		for i := 0 to pred(bitmap.Width) do begin
			if pSour16^ <> colorKey then
				pDest16^ := pSour16^;
			inc(pSour16);
			inc(pDest16);
		end;
	end;
end;

procedure TFrameBuffer.Flush(const canvas: TCanvas; x, y: integer);
begin
	canvas.Draw(x, y, Self);
end;

end.
 